# -*- coding: utf-8 -*-
import json
import os
def htmlFormater(dataList):
    """


    """
    
    headArray=[]
    bodyArray=[]
    for elem in dataList:

        if elem["type"] == "title":
            res=htmlFormat_title(elem)

            headArray.append(res)
        else:

            res=HTMLFORMAT_DIC[elem["type"]](elem)
            bodyArray.append(res)

    headText = u"<head> %s  </head>"%(u" ".join(headArray))
    
    bodyText = u"<body>%s</body>"%(u" ".join(bodyArray),)



    return u"""<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
%s 
%s
</html>"""%(
            headText,
            bodyText
            )

def htmlInlineFormater(dataList):
    """

    """
    bodyArray=[]
    for elem in dataList:
        if elem["type"] == "title":
            pass
        else:
            res=HTMLFORMAT_DIC[elem["type"]](elem)
            bodyArray.append(res)
    return u" ".join(bodyArray)

def htmlFormat_table(elem):
    """
    """
    import pandas as pd
    if elem["format"] == "pd.frame":
        return elem["data"].to_html()
    else:
        raise Exception("elem format not supported %s"%(elem["format"]))
def htmlFormat_title(elem):
    """
    """
    return u"<title>%s</title>"%elem["text"]
def htmlFormat_h1(elem):
    """
    """
    return u"<h1>%s</h1>"%elem["text"]
def htmlFormat_h2(elem):
    """
    """
    return u"<h2>%s</h2>"%elem["text"]
def htmlFormat_img(elem):
    """
    """

    if elem["format"] == "png":

        return u"""
    <img alt="Embedded Image" 
        src="data:image/png;base64,%s" />
"""%(elem["data"])

def htmlFormat_code(elem):
    """
    """
    return u"<pre>%s</pre>"%elem["text"]

def htmlFormat_p(elem):
    """
    """
    return u"<p>%s</p>"%elem["text"]


HTMLFORMAT_DIC = {
        "h1":htmlFormat_h1,
        "h2":htmlFormat_h2,
        "title":htmlFormat_title,
        "img":htmlFormat_img,
        "p":htmlFormat_p,
        "code":htmlFormat_code,
        "table":htmlFormat_table,

        }







class JsonEscaper(json.JSONEncoder):
    def default(self,obj):
        return u"ErrorJSON"

def pdframetotable_bs(df):
    from bs4 import BeautifulSoup
    soup = BeautifulSoup(df.to_html(), 'html.parser')
    table = soup.find_all("table")[0]

    res = soup.find_all("table")[0]
    newclassset = res["class"]+['table panel-body']

    res["class"] = newclassset
    return soup.prettify()


def htmlBootstrapFormater(dataList):
    """
    """
    from jinja2 import Template,Environment,PackageLoader
    #print os.path.realpath(__file__)
    env=Environment(loader=PackageLoader('nadej','templates'))
    
    env.filters['pdframetotable'] = pdframetotable_bs
    template = env.get_template('bootstraped.html')
    
    djson = json.dumps(dataList,indent=2,cls=JsonEscaper)
    
    title = u"No Title"

    for elem in dataList:
        if elem["type"] == "title":
            title=elem["text"]


    meta={
            "title":title,
            "lead":u""
            }
    return template.render(d=dataList,djson=djson,meta=meta)









import argparse
import json

import nadej

def elementMatcher(elem):
    """
    """
    if elem["keyword"] == "Feature":
        return feature(elem)
    elif elem["keyword"] == "Scenario":
        return scenario(elem)
    elif elem["keyword"] == "Scenario Outline":
        return scenarioOL(elem) 
    elif elem["keyword"] in ["Given","Then","And"]:
        return given_then(elem) 
    else:
        raise Exception ("keyword not supported : %s"%elem['keyword'])

def feature(feat):
    resArray= []

    nadej.h1(feat["name"])
    nadej.text(u"in %s"%feat["location"])
    nadej.text(u"tags : %s"%feat["tags"])
    
    for elem in feat["elements"]:
        elementMatcher(elem)
    print feat.keys()
def scenario(elem):
    nadej.h2(elem["name"])
    nadej.text(u"sc in %s"%elem["location"])
    for sub in elem["steps"]:
        elementMatcher(sub)

def given_then(elem):
    nadej.text(elem["name"])


def scenarioOL(elem):
    nadej.h2(elem["name"])
    nadej.text(u"sc in %s"%elem["location"])
    for sub in elem["steps"]:
        elementMatcher(sub)

def main(args):
    """
    """
      
    with open(args.inputfile) as inputfile:
        injson =json.load(inputfile)

    
    for elem in injson:
        elementMatcher(elem)
    inhtml = nadej.collect("html_bs:save in %s"%args.output)

    # print 

if __name__=="__main__":
    parser = argparse.ArgumentParser(description='Convert Behave json output to html')
    parser.add_argument('inputfile',  type=str ,help='input json')
    parser.add_argument('output', type=str ,help='destination')
    parser.add_argument('-f', type=str ,help='option')

    args = parser.parse_args()

    main(args)




import os
from StringIO import StringIO
from  htmlout import htmlInlineFormater,htmlBootstrapFormater,htmlFormater


def savein(data,dest):
    """

    save data in html
    """
    if os.path.isabs(dest) :
        return
    
    
    if dest.endswith(".html"):
        with open(dest,"w") as ou:
            ou.write(data)
    else:
        raise Exception ("Format not accepted")
class ClientAPI(object):

    def __init__(self):
        self.dataList=[]

    def title(self,text=""):
        """
        Clean up buffer
        """
        self.dataList=[]
        dic = {"type":"title",
                "text":text}
        self.dataList.append(dic)

    def h1(self,text=""):
        """
        """

        dic = {"type":"h1",
                "text":text}
        self.dataList.append(dic)
        

    def h2(self,text=""):
        """
        """
        dic = {"type":"h2",
                "text":text}
        self.dataList.append(dic)

    def json(self,text=""):
        dic = {"type":"code",
                "format":"json",
                "text":text}
        self.dataList.append(dic)

    def text(self,text=""):
        """
        """
        dic = {"type":"p",
                "text":text}
        self.dataList.append(dic)

    def table(self,data=""):
        """
        """
        import pandas as pd
        dic = {"type":"table",
                "format":"pd.frame",
                "data":data
                }
        self.dataList.append(dic)
    def png(self,data):
        """
        data should be base64
        """
        dic = {"type":"img",
               "format":"png",
               "data":data}
        self.dataList.append(dic)


    def plot(self,data=""):
        """
        """
        import pandas as pd

        if isinstance(data,pd.DataFrame):
            import matplotlib
            matplotlib.use('Agg')
            import matplotlib.pyplot as plt
            plt.plot(data)
            
            plt.legend(data.columns)

            bufferIMG = StringIO()
            plt.savefig(bufferIMG)
            from base64 import b64encode,b64decode

            buffer64 = b64encode(bufferIMG.getvalue())

            self.png(buffer64)

        else:
            raise Exception("not supported yet")

    def collect(self,outpipe=""):
        """
        """
        ret = self.dataList
        self.dataList=[]

        if outpipe == "":
            return ret
        
        pipe = outpipe.split(":")
        for step in pipe:
            if step == "html":
                ret= htmlFormater(ret)
            elif step == "htmlinline":
                ret= htmlInlineFormater(ret)
            elif step == "html_bs":
                
                
                ret= htmlBootstrapFormater(ret)
            elif step.startswith("save in "):
                dest = step.split(" ")[-1]
                savein(ret,dest)

            else:
                raise Exception("pipe step not found: %s"%step)

        return ret

